@@ -17,9 +17,15 @@
 /* Default runtime library to use. */
 #define CLANG_DEFAULT_RTLIB ""
 
+/* Default objcopy to use */
+#define CLANG_DEFAULT_OBJCOPY "objcopy"
+
 /* Default OpenMP runtime used by -fopenmp. */
 #define CLANG_DEFAULT_OPENMP_RUNTIME "libomp"
 
+/* Default architecture for OpenMP offloading to Nvidia GPUs. */
+#define CLANG_OPENMP_NVPTX_DEFAULT_ARCH "sm_35"
+
 /* Multilib suffix for libdir. */
 #define CLANG_LIBDIR_SUFFIX ""
 
@@ -29,6 +35,10 @@
 /* Directories clang will search for headers */
 #define C_INCLUDE_DIRS ""
 
+/* Directories clang will search for configuration files */
+/* #undef CLANG_CONFIG_FILE_SYSTEM_DIR */
+/* #undef CLANG_CONFIG_FILE_USER_DIR */
+
 /* Default <path> to all compiler invocations for --sysroot=<path>. */
 #define DEFAULT_SYSROOT ""
 
@@ -45,7 +55,7 @@
 #define CLANG_HAVE_RLIMITS 1
 
 /* The LLVM product name and version */
-#define BACKEND_PACKAGE_STRING "LLVM 5.0.1"
+#define BACKEND_PACKAGE_STRING "LLVM 6.0.1"
 
 /* Linker version detected at compile time. */
 /* #undef HOST_LINK_VERSION */
@@ -57,8 +67,8 @@
 #define ENABLE_X86_RELAX_RELOCATIONS 0
 
 /* Enable each functionality of modules */
-#define CLANG_ENABLE_ARCMT
-#define CLANG_ENABLE_OBJC_REWRITER
-#define CLANG_ENABLE_STATIC_ANALYZER
+#define CLANG_ENABLE_ARCMT 1
+#define CLANG_ENABLE_OBJC_REWRITER 1
+#define CLANG_ENABLE_STATIC_ANALYZER 1
 
 #endif
