@@ -14,11 +14,14 @@
 #ifndef LLVM_CONFIG_H
 #define LLVM_CONFIG_H
 
+/* Define if LLVM_ENABLE_DUMP is enabled */
+/* #undef LLVM_ENABLE_DUMP */
+
 /* Define if we link Polly to the tools */
 /* #undef LINK_POLLY_INTO_TOOLS */
 
 /* Target triple LLVM will generate code for by default */
-#define LLVM_DEFAULT_TARGET_TRIPLE "i386-unknown-openbsd6.3"
+#define LLVM_DEFAULT_TARGET_TRIPLE "aarch64-unknown-openbsd6.4"
 
 /* Define if threads enabled */
 #define LLVM_ENABLE_THREADS 1
@@ -27,28 +30,28 @@
 #define LLVM_HAS_ATOMICS 1
 
 /* Host triple LLVM will be executed on */
-#define LLVM_HOST_TRIPLE "i386-unknown-openbsd6.3"
+#define LLVM_HOST_TRIPLE "aarch64-unknown-openbsd6.4"
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH X86
+#define LLVM_NATIVE_ARCH AArch64
 
 /* LLVM name for the native AsmParser init function, if available */
-#define LLVM_NATIVE_ASMPARSER LLVMInitializeX86AsmParser
+#define LLVM_NATIVE_ASMPARSER LLVMInitializeAArch64AsmParser
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializeX86AsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializeAArch64AsmPrinter
 
 /* LLVM name for the native Disassembler init function, if available */
-#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeX86Disassembler
+#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeAArch64Disassembler
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializeX86Target
+#define LLVM_NATIVE_TARGET LLVMInitializeAArch64Target
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializeX86TargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializeAArch64TargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializeX86TargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializeAArch64TargetMC
 
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
@@ -63,7 +66,7 @@
 #define LLVM_USE_OPROFILE 0
 
 /* Major version of the LLVM API */
-#define LLVM_VERSION_MAJOR 5
+#define LLVM_VERSION_MAJOR 6
 
 /* Minor version of the LLVM API */
 #define LLVM_VERSION_MINOR 0
@@ -72,6 +75,6 @@
 #define LLVM_VERSION_PATCH 1
 
 /* LLVM version string */
-#define LLVM_VERSION_STRING "5.0.1"
+#define LLVM_VERSION_STRING "6.0.1"
 
 #endif
